/* 
    Battmon.c
    
    Linuxbot utility to read the battery voltage and, if the voltage
    has fallen below a given threshold value, initiate a shutdown.
    
    This program may be run from the command line (as root or su), to
    print out the current battery voltage (and possibly shutdown).
    
    However, battmon is really intended to be run as a 'cron' job so
    that every, say, 5 minutes, battmon is run in the background.  Thus
    relieving the robot user from the burden of worrying about when the
    robot's battery is about to expire.

    To install battmon as a cron job, login as root, then use 'crontab -e'
    to edit the following line into the crontab...
      0-59/5 * * * * /home/robot/battmon > /home/robot/battlog

    This will run battmon every 5 minutes, and pipe its output to 'battlog'.
    Also add 
      /home/robot/stop 
    to file rc.local so that the analog i/o is initialised after power-up.

    Alan FT Winfield and Etienne Falaise
    April 2001.
*/

#include <stdio.h>

#define LINUXBOT
#include "robot.c"

/* Enter the minimum battery voltage here... */
#define LOWBATT 10.8

int main( void )
{
	int i;
	float battsum = 0.0, battavg;
	robot Bot;

	/* request access to i/o ports.. */
	if ( !accessPorts() )
	{
		printf("Port access denied..  root privileges needed\n");
		exit(1);
	}
	
	/* Do not call resetBot() here, since that would stop the motors
	   which may be under the control of another process. */
	ticksperusec = calibrate();

	/* sample the battery voltage over one second */
	for (i=0; i<5; i++)
	{
		battsum += getBattery();
		usleep( 200000 );	/* wait 200mS */
	}
	
	battavg = battsum/5.0;
	printf( "Average battery voltage = %2.2f\n", battavg );
	
	if (battavg < LOWBATT )
	{
		/* Shutdown... */
		resetBot( &Bot );	/* reset motors and open loop stop */
		/* should also disable the motors here */
		
		system( "shutdown -h +1 !!! BATTERY LOW !!! ");
	}
}
