/*-----------------------------------------------------------------------------

   ROBOT.H

   RASCAL/LINUXBOT/MOOREBOT HEADER for

   CONTROL FUNCTIONS LIBRARY FOR RASCAL/LINUXBOT/MOOREBOT

   Copyright (c) 2000 Intelligent Autonomous Systems Laboratory
   University of the West of England, Bristol

   author  Alan Winfield
   email   Alan.Winfield@uwe.ac.uk

   V1.8
   for revision history see ROBOT.C

   Usage:
   BEFORE INCLUDING THIS FILE the programmer MUST define either RASCAL,
   LINUXBOT or MOOREBOT, otherwise the compiler will fail.  Ie,
   #define RASCAL 
   #include "robot.h"

   or,
   #define LINUXBOT 
   #include "robot.h"

   etc.
-----------------------------------------------------------------------------*/

/*--RASCAL defines-----------------------------------------------------------*/
#ifdef	RASCAL

/* RASCAL addresses... */
#define BASE_ADDR  		0x220			/* all is referenced from here */

/* HCTL1100 Motion Controller addresses.. */
#define LEFT_MCTRL		BASE_ADDR  		/* LEFT motor controller */
#define RIGHT_MCTRL 		BASE_ADDR+1       	/* RIGHT motor controller */
#define LEFT_MCTRL2		BASE_ADDR+2     	/* LEFT, 2nd read address */
#define RIGHT_MCTRL2		BASE_ADDR+2+(1<<10)	/* RIGHT, 2nd read address */

/* Other RASCAL port addresses.. */
#define LED_PORT		BASE_ADDR+3		/* LED port (write only) */
#define STATUS_PORT		BASE_ADDR+4		/* STATUS port (read only) */
#define ENABLE_PORT		BASE_ADDR+5		/* motor enable, etc */
#define ADC_READ		BASE_ADDR+6		/* ADC read port */
#define ADC_WRITE		BASE_ADDR+7		/* ADC write port */

/* Battery voltage multiplier on A/D channel 0 input.. */
#define BATT_SCALE		5.68

#endif	/* RASCAL */
/*---------------------------------------------------------------------------*/

/*--LINUXBOT or MOOREBOT defines---------------------------------------------*/
#ifdef MOOREBOT
#define LINUXBOT 1
#endif

#ifdef LINUXBOT

/* LinuxBot/Moorebot addresses... */
#define BASE_ADDR  		0x220			/* all is referenced from here */

/* HCTL1100 Motion Controller addresses.. */
#define LEFT_MCTRL		BASE_ADDR+1  		/* LEFT motor controller */
#define LEFT_MCTRL2		BASE_ADDR+2     	/* LEFT, 2nd read address */
#define RIGHT_MCTRL 		BASE_ADDR+4       	/* RIGHT motor controller */
#define RIGHT_MCTRL2		BASE_ADDR+5		/* RIGHT, 2nd read address */

/* Other LinuxBot/Moorebot port addresses.. */
#define SPARE2			BASE_ADDR		/* spare port decode */
#define IPLATCH2		BASE_ADDR+3		/* Input latch 2 */
#define ADC_WRITE		BASE_ADDR+6		/* ADC write port */
#define ADC_READ_LOW		BASE_ADDR+7		/* ADC read port - low byte */
#define IPLATCH1		BASE_ADDR+8		/* Input latch 1 */
#define ADC_READ_HIGH		BASE_ADDR+9		/* ADC read port - high byte */
#define DAC1_WRITE		BASE_ADDR+10		/* DAC 1 write port */
#define OPLATCH1		BASE_ADDR+11		/* Output latch 1 */
#define DAC2_WRITE		BASE_ADDR+12		/* DAC 2 write port */
#define PWRD			BASE_ADDR+13		/* Current monitor */
#define SPARE1			BASE_ADDR+14		/* spare port decode */
#define OPLATCH2		BASE_ADDR+15		/* Output latch 2 */

/* Battery voltage multiplier on A/D channel 0 input.. */
#define BATT_SCALE		5.58

#endif	/* LINUXBOT */
/*---------------------------------------------------------------------------*/


/* HCTL1100 Motion Controller registers... */
#define FLAG_REG		0  			/* Flag register */
#define PC_REG 			5  			/* Program Counter register */
#define STATUS_REG		7			/* Status register */

#define PWM_MOTOR_PORT		9			/* PWM Motor Command port */

#define COMMAND_POS_REG_MSB	0x0C			/* Command position MSB */
#define COMMAND_POS_REG		0x0D			/* Command position */
#define COMMAND_POS_REG_LSB	0x0E			/* Command position LSB */

#define SAMPLE_TIMER_REG	0x0F			/* Sample timer */

#define ACTUAL_POS_REG_MSB	0x12			/* Read actual position MSB */
#define ACTUAL_POS_REG		0x13			/* Read actual position */
#define ACTUAL_POS_REG_LSB	0x14			/* Read actual position LSB */

#define	FILTER_ZERO_REG		0x20 			/* Filter Zero A register */
#define FILTER_POLE_REG 	0x21 			/* Filter Pole B register */
#define FILTER_GAIN_REG 	0x22 			/* Filter Gain register */

#define COMMAND_VEL_REG_LSB	0x23			/* Prop vel mode Command velocity LSB */
#define COMMAND_VEL_REG_MSB	0x24			/* Prop vel mode Command velocity MSB */

#define	ACCEL_REG_LSB		0x26			/* Acceleration LSB */
#define ACCEL_REG_MSB		0x27			/* Acceleration MSB */

#define MAX_VEL_REG		0x28			/* Trap mode Maximum Velocity */
#define FINAL_POS_REG_LSB	0x29			/* Trap mode Final position LSB */
#define FINAL_POS_REG		0x2A			/* Trap mode Final position */
#define FINAL_POS_REG_MSB	0x2B			/* Trap mode Final position MSB */

#define	INT_COMMAND_VEL_REG	0x3C			/* Integral mode Command velocity */

/* HCTL1100 FLAG register values.. */
#define F0_CLR			0      			/* clear trapezoidal profile mode */
#define F2_CLR      		2			/* clear unipolar mode */
#define F3_CLR   		3			/* clear proportional velocity mode */
#define F4_CLR   		4			/* clear hold commutator flag */
#define F5_CLR   		5			/* clear integral velocity mode */
#define F0_SET			8      			/* set trapezoidal profile mode */
#define F2_SET		      	0xA			/* set unipolar mode */
#define F3_SET   		0xB			/* set proportional velocity mode */
#define F4_SET   		0xC 			/* set hold commutator flag */
#define F5_SET   		0xD			/* set integral velocity mode */


/* ROBOT Physical characteristics... */
#define SLOTS_PER_REV		500 			/* slots in encoder's codewheel */
#define WHEEL_DIAMETER		0.064			/* wheel diameter in metres */
#define WHEEL_BASE		0.231			/* Rascal wheel base (width) */
#define GEARBOX_RATIO		(50.0/3.0)		/* gearbox step down ratio */
#define PI 			(3.141592653589793116)  /* pi */
#define COUNTS_PER_REV		(4*SLOTS_PER_REV*GEARBOX_RATIO)   /* Note: the number of */
							/* quadrature encoder counts is 4 * the */
                                                        /* number of slots in encoder's codewheel */
#define COUNTS_PER_METRE	(COUNTS_PER_REV/(PI*WHEEL_DIAMETER))
#define CLOCK_FREQ		2000000			/* HCTL1100 clock freq = 2MHz */


/* ROBOT OPERATING MODES.. */
/* note: the actual values are meaningless, but chosen to be unlikely to be confused with the
         contents of an uninitialised variable */
#define IDLE_MODE		1234		/* Idle Mode (also open loop) */
#define PC_MODE			2345		/* Position Control Mode */
#define PV_MODE			3456		/* Proportional Velocity Mode */
#define IV_MODE			4567		/* Integral Velocity Mode */
#define TP_MODE			5678		/* Trapezoid Profile Mode */
#define UNDEFINED_MODE		6789		/* none of the above */

/* FUNCTION RETURNS.. */
#define PORT_ACCESS_OK	    	1		/* Linux denied port access */
#define WRONG_MODE_ERROR    	2		/* Wrong mode for this operation */

/* ROBOT DEFAULT VALUES.. */
#define DEFAULT_T	  	64		/* default sample time in all modes */
#define DEFAULT_K              	64		/* default filter gain */
#define DEFAULT_A	  	229		/* default filter zeroes */
#define DEFAULT_B	  	64		/* default filter poles */
#define DEF_SAMPLE_TIME	  	(16.0*(DEFAULT_T+1)/CLOCK_FREQ)	/* DEFAULT_T in seconds */

#define DEFAULT_MAXMPS	  	0.5		/* default max m/s in trap moves */
#define DEFAULT_MPSS	  	0.8		/* default m/s/s in trap etc moves */


/* Data structures start here... */
typedef unsigned char byte;

typedef struct {
	byte 	T;                	/* Digital filter parameters... */
        byte 	K;
        byte 	A;
        byte 	B;
        double 	sampleperiod;	       	/* T expressed in seconds */

        int	mode;     		/* current mode, see OPERATING MODES above */

        float 	maxmps;			/* maximum velocity in m/s, TP mode only */
        float	mpss;                 	/* acceleration in m/s/s, TP and IV modes */
} robot;

